# This script does the preprocessing of
# creating the intial values, reads and formats the observational data on prevalences in food and human cases,
# starts OpenBUGS and finally writes the results to disk and visualizes them


library(R2OpenBUGS) ## needed for the use of openbugs
library(lattice)    ## needed for my kind of plots
library(coda)       ## needed to access and use CODA objects
library(openxlsx)       ## to lead xlsx tabels


#--------------------------------------------------------------------------------------------
# Define functions

# reads an Excel file (*.csv) into R, formats the data
# and writes data to text file so OpenBugs is able to read it

format_data = function(n.sources, n.subtypes, n.OS, input_file){
  dimension = c(n.sources, n.subtypes)
  
  # combine prevalences into one vector
  p = c()
  for (i in 1:n.sources)
  {
    p = c(p, as.numeric(input_file[[i]]))
  }
  
  p = formatC(p, format = "E")
  
  # write data to text file so OpenBugs is able to read it
  output <-list( o = c(input_file[[n.sources+1]]), # print observed cases o
                 m = c(input_file[[n.sources + 2]][1:n.sources]), # print consumption data m
                 n.subtypes =n.subtypes,
                 n.OS = n.OS,
                 n.sources = n.sources, 
                 p = matrix(as.numeric(p),nrow =n.sources, ncol = n.subtypes, byrow = T)
                 #   structure(.Data = as.numeric(p),# print prevalences p
                 # .Dim = dimension)
  )
  
  return(output)
  
}           

# Change hirarchical inits
change_inits_uniform = function(n.sources,n.subtypes,a.value,q.value,na.ind,qfix_ind,afix.ind,cutoff)
{
  
  a = rep(a.value, times=n.sources)
  a[afix.ind] = NA
  
  q = rep(q.value, times=n.subtypes)
  q[na.ind] = NA
  q[qfix_ind] = NA
  q = q[1:(n.subtypes-cutoff)]
  
  init = list(a = a,q = q)
  print(init)
  return(init)
}


#--------------------------------------------------------------------------------------------
# Writing and formatting the intial values data

DataSet <- read.csv(input_FileName, sep=';', na.strings = c('<NA>', 'NA' ), header = T)# name of the file with the data

AvailData <- c(sub(replacement = '.',pattern = ' ', x = list_sources),
               'Observed.human.cases',
               'Consumption')

input_file <- DataSet[,sapply(AvailData,function(i.iter) which(i.iter==colnames(DataSet)))]

na.ind = grep(pattern = 'other',DataSet$Subtype,ignore.case = T, value=F ) # indices, of subtypes "others" (see also variable "cutoff")
afix.ind = c() # Indices of the a_j, which will be set to fixed values

cutoff = length(na.ind) # number of NAs, which we put at the end of the vectors which contain the data for types. This number of NAs values correspond to the "other types" (see also vairable "na.ind")
n.sources = length(list_sources) # number of sources
n.subtypes = length(unique(DataSet$Subtype)) # number of subtypes
n.OS = as.integer(na.ind[1]-1) # gives the index up to which we have subtypes (without "other" subtypes)

# All five sets of intial values are put together in one list
## create sets of inital values (each for one MCMC chain) according to a_j~uniform(0,0.2) und q_i~uniform(0,0.2)
OpenBUGS_inits = lapply(1:length(aValue), function(i.iter){
  temp.inits <-change_inits_uniform(n.sources,n.subtypes,a.value = aValue[i.iter],q.value = qValue[i.iter],na.ind,qfix_ind, afix.ind,cutoff)
})

#--------------------------------------------------------------------------------------------
# Reading and formatting the observational data

OpenBUGS_Data <-format_data(n.sources, n.subtypes,n.OS,input_file) # function which reads, formats and writes the formatted data

#--------------------------------------------------------------------------------------------------
# Running  OpenBUGS

# calling OpenBUGS 
bugs_Diag <- bugs(
  data = OpenBUGS_Data, # define data for model
  inits=OpenBUGS_inits, # provide inital values
  parameters.to.save = OpenBUGS_parameter, # define which values should be logged
  model.file = OpenBUGS_model, # define file where the BUGS model code is stored
  n.chains=length(OpenBUGS_inits), # number of MCMC chains
  n.iter=OpenBUGS_niter, # number of iterations
  n.burnin = OpenBUGS_nburnin, # number of burn-in iterations
  # OpenBUGS.pgm = bugsdir, # directory where I have installed openbugs
  debug=F# TRUE: OpenBugs will be opened in a window, FALSE: OpenBugs just runs in the background
  #working.directory = workdir
) # 

# save results from openbugs into variable "bugssim"
bugssim = bugs_Diag

# save model results into a "*.Rdata"-file
#save(bugssim, file= paste0('OpenBUGS_bugssim_', sub(pattern = '.xlsx', replacement = '', x = input_FileName), ".Rdata"))

#--------------------------------------------------------------------------------------------------
# Show the OpenBUGS results in console and as visualizations

# get the (four) means, for the model estimated number of human cases attributed to the four sources
source.sim = bugssim$mean$source

# estimated number of cases due to unknown sources 
unknown.sim = bugssim$mean$unknown

# combine results for the four considered sources and unknown sources in one matrix
quellen.sim = c(source.sim, unknown.sim)

# calulate which proportion of the total human salmonellosis cases  are attributable to the sources
p.quellen.sim = quellen.sim/sum(quellen.sim)

# 
# sink( paste0('OpenBUGS_results_', sub(pattern = '.xlsx', replacement = '', x = input_FileName), ".txt"), split=TRUE) # writes results into file "my_results.txt"
# 
# cat("sum of all human cases = ", sum(quellen.sim), "\n") #show number of estimated total number of human cases

# Calculate proportion of nuber of cases if one excludes cases due to unknown sources
p.source = source.sim/sum(source.sim)

# get the estimated mean values of the lambdaexp parameter (i.e. the number of expected human cases)
lambdaexp.means = bugssim$mean$lambdaexp

## Values from Table 5 r 6 in the Paper are
## bugssim$mean$source
## bugssim$sd$source
## bugssim$mean$unknown
## bugssim$sd$unknown

## the same can be calculated by hand

mean_broilers <- mean(bugssim$sims.list$source[,1])                         
CI_broilers <- quantile(bugssim$sims.list$source[,1], c(.05,.95))          

mean_laying_hens <- mean(bugssim$sims.list$source[,2])                   
CI_laying_hens <- quantile(bugssim$sims.list$source[,2], c(.05,.95))    

mean_pigs <- mean(bugssim$sims.list$source[,3])                           
CI_pigs <- quantile(bugssim$sims.list$source[,3], c(.05,.95))          

mean_turkeys <- mean(bugssim$sims.list$source[,4])                        
CI_turkeys <- quantile(bugssim$sims.list$source[,4], c(.05,.95))           

mean_unknown <- mean(bugssim$sims.list$unknown)                          
CI_unknown <- quantile(bugssim$sims.list$unknown, c(.05,.95))            

mean_res=c(mean_broilers, mean_laying_hens, mean_pigs, mean_turkeys, mean_unknown)


## By the way, are "equail tailed CI" OK? I.e., are the distributions symmetric? Yes, look here:

## hist(bugssim$sims.list$source[,1])
## hist(bugssim$sims.list$source[,2])
## hist(bugssim$sims.list$source[,3])
## hist(bugssim$sims.list$source[,4])
## hist(bugssim$sims.list$unknown)

# get 5% quantil of human cases for each source
quantil_05 <- c(unname(CI_broilers[1]), 
                unname(CI_laying_hens[1]),
                unname(CI_pigs[1]),
                unname(CI_turkeys[1]),
                unname(CI_unknown[1])) 

# get 95% quantil of human cases for each source
quantil_95 <- c(unname(CI_broilers[2]), 
                unname(CI_laying_hens[2]),
                unname(CI_pigs[2]),
                unname(CI_turkeys[2]),
                unname(CI_unknown[2])) 


# # Define output
#   mean_res=c(mean_broilers, mean_laying_hens, mean_pigs, mean_turkeys, mean_unknown)
#   quantil_95
#   quantil_05
